package com.aspirecn.jiaoke.data.extract

import java.sql.Date

import com.aspirecn.jiaoke.data.extract.DataExtractMonth.{TrcExetcpu, exists}
import org.apache.log4j.{Level, Logger}
import org.apache.spark.sql.{DataFrame, SparkSession}

/**
 * @auther: hwj
 * @date: 2021/4/25 13:48
 * @description:
 */
object DataExtractMonthMerge {
  case class monthResult(month:String,vehicleid:String,max_exTime:Date,sum_travel_time:String,max_travel_time:String,sum_feemileage:String,sum_fee:String,sum_weight_mileage:String,exTimes_count:String,axlecount:String,vehicletype:String,travel_provinces_count:String,transtime_count:String)
  def main(args: Array[String]): Unit = {
    val log: Logger = Logger.getLogger("DataExtractMonthMerge")
    log.setLevel(Level.INFO)
    if (args.size != 5){
      log.error(
        """
          |请正确传入程序参数：
          |                  1.1——10号数据读取目录
          |                  2.11——20号数据读取目录
          |                  3.21——31号数据读取目录
          |                  4.结果文件的输出路径（文件输出目录在执行不可存在，程序会自动进行创建，否则会报错）
          |                  5.结果文件输出个数，请根据数据量合理输入该参数，输出的文件越少，执行时间越长，输入0则为默认文件个数
          |""".stripMargin)
      System.exit(0)
    }
    val month_1:String =args(0)
    val month_2:String =args(1)
    val month_3:String =args(2)
    val out_path:String =args(3)
    val result_partitions:Int = Integer.valueOf(args(4))
    val spark:SparkSession = formatSparkSession
    // 当结果数据输出目录已存在时，结束程序
    if(exists(out_path)){
      log.error("数据输出目录已存在"+out_path)
      System.exit(0)
    }

    //todo：开始处理数据
    import org.apache.spark.sql.functions._
    // todo: 引入隐式转换
    import spark.implicits._
    val trcMonthDataSet = spark.read.option("header", true).csv(month_1,month_2,month_3).as[monthResult]
    trcMonthDataSet.createOrReplaceTempView("trcMonthTable")
//    val provinceFrame:DataFrame = spark.sql(
//      """
//        |select
//        |month,
//        |vehicleid
//        |str_to_map(travel_provinces_count,'#','%') as travel_provinces_count,
//        |str_to_map(transtime_count,'#','%') as transtime_count
//        |from
//        |trcMonthTable
//        |""".stripMargin)
//    provinceFrame.createOrReplaceTempView("provinceFrame")
    val provinceFrame = spark.sql(
      """
        |select
        |a.month as month,
        |a.vehicleid as vehicleid,
        |a.travel_province as travel_province,
        |sum(a.travel_count) as travel_count
        |from
        |(select
        |month,
        |vehicleid,
        |split(travel_province_count,'%')[0] as travel_province,
        |split(travel_province_count,'%')[1] as travel_count
        |from
        |trcMonthTable
        |lateral view explode(split(travel_provinces_count, '#')) as travel_province_count ) as a
        |group by a.month,a.vehicleid,a.travel_province
        |
        |""".stripMargin)
    val transtimeFrame = spark.sql(
      """
        |select
        |a.month,
        |a.vehicleid,a.transtime,
        |sum(a.transtime_c) as transtime_c
        |from
        |(select
        |month,
        |vehicleid,
        |split(transtime_count_n,'%')[0] as transtime,
        |split(transtime_count_n,'%')[1] as transtime_c
        |from
        |trcMonthTable
        |lateral view explode(split(transtime_count, '#')) as transtime_count_n ) a
        |group by a.month,a.vehicleid,a.transtime
        |
        |""".stripMargin)
    provinceFrame.createOrReplaceTempView("provinceFrame")
    transtimeFrame.createOrReplaceTempView("transtimeFrame")
    val travel_provinces_countFrame = spark.sql(
    """
      |select month,vehicleid,
      |concat_ws("#",collect_list(concat_ws("%",travel_province,cast(travel_count  as int)) )) as travel_provinces_count
      |from
      |provinceFrame
      |group by month,vehicleid
      |""".stripMargin)
    val transtimecountFrame = spark.sql(
    """
      |select month,vehicleid,
      |concat_ws("#",collect_list(concat_ws("%",transtime,cast(transtime_c as int)) )) as transtime_count
      |from
      |transtimeFrame
      |group by month,vehicleid
      |""".stripMargin)
    travel_provinces_countFrame.show(10)
    transtimecountFrame.show(10)
    val  travelProvincesTranstimeCountDataFrame: DataFrame = travel_provinces_countFrame.join(transtimecountFrame,
      Seq("month", "vehicleid"),
      "outer")
    val vehicleidDataFrame:DataFrame = spark.sql(
    """
        |select
        |month, vehicleid,
        |max(max_exTime) as max_exTime,
        |sum(sum_travel_time) as sum_travel_time,
        |max(max_travel_time) as max_travel_time,
        |sum(sum_feemileage) as sum_feemileage,
        |sum(sum_fee) as sum_fee,
        |sum(sum_weight_mileage) as sum_weight_mileage,
        |sum(exTimes_count) as exTimes_count,
        |max(axlecount) as axlecount,
        |max(vehicletype) as vehicletype
        |from
        |trcMonthTable
        |group by month, vehicleid
        |""".stripMargin)
    vehicleidDataFrame.show(10)
    var trcResultDataFrame = vehicleidDataFrame.join(travelProvincesTranstimeCountDataFrame,
      Seq("month", "vehicleid"),
      "outer")
    trcResultDataFrame.show(10)
    trcResultDataFrame.printSchema()
    if (result_partitions> 0){
      trcResultDataFrame= trcResultDataFrame.repartition(result_partitions)
    }
    trcResultDataFrame.write.option("header",true).csv(out_path)


    spark.stop()

  }
  def formatSparkSession(): SparkSession ={
    // todo: 实例化SparkSession
    val spark: SparkSession = SparkSession
      .builder()
      //   .appName("DataExtract")
    //  .master("local[*]")
      .getOrCreate()
    spark
  }
}
